/*
 *  connpalette
 */
#include "copyright.h"
#include "port.h"
#include "oct.h"
#include "oh.h"
#include "symlib.h"
#include "errtrap.h"
#include "options.h"

octObject	InstanceBag;
octObject	ConnectorBag;
octObject	EditStyle;
octObject	ViewType;
octObject	Technology;
octObject	TapView;
octObject	TapInstance;
octObject 	Palette;



char    	*progName;
int		verbose = 0;


char 		buffer[256];

optionStruct optionList[] = {
    {"f", "conn file name",    "file with technological info" },
    {"t", "technology name",   "defaults to scmos" },
    {"c", "view",	       "default view for connectors (physical)"},
    {"o", "palette view",      "defaults to symbolic" },
    { 0, 0 , 0}
};


main(argc,argv)
    int argc;
    char *argv[];
{
    octObject	Instance;
    octObject   bag;		/* PALETTES bag. */
    char	*technology = "scmos";
    char        *paletteview = "symbolic";
    char        *connview = "physical";
    char	*filename = 0;
    char	pname[256]; /* Palette name */
    char	tname[256]; /* Tap view name */
    char	buffer[256];
    char	input[256];
    FILE 	*fp;
    int		i = 0;
    int         c;
    extern int optind;
    extern char* optarg;
    struct octBox Box;
    octCoord	padding = 80;
    octCoord	rowHeight = 0;
    octCoord    height = 0;
    octCoord	x = 0;
    octCoord	y = 0;

    /* parse command line options*/

    while ( (c = optGetOption(argc,argv ) )!= EOF ) {
	switch ( c ) {
	case 'f': filename = optarg; break;
	case 'c': connview = optarg; break;
	case 't': technology = optarg; break;
	case 'o': paletteview = optarg; break;
	default: optUsage();
	}
    }
    if ( argc < 2 ) {
	optUsage();
    }

    if ( !filename ) {
	errRaise("XX",1, "You must specify the name of a file (-f option)\n\n" );
    }

    if ((fp = fopen(filename,"r")) == NULL) {
	errRaise("XX", 1, "Cannot open file %s",filename);
    }

    sprintf(pname,"$OCTTOOLS/tech/%s/connector-palette", technology);
    sprintf(buffer, "%s:%s:contents", pname, paletteview  );

    ohUnpackFacetName( &Palette, buffer );
    symOpenFacet( &Palette, "w", technology );


    printf("Recreating %s\n", ohFormatName( &Palette ));
  
    OH_ASSERT(ohGetOrCreateBag(&Palette,&InstanceBag, "INSTANCES"));
    OH_ASSERT(ohGetOrCreateBag(&Palette,&ConnectorBag, "CONNECTORS"));


    while (1) {
  
	if (fscanf(fp,"%s", input) == EOF) break;
  
	Instance.type = OCT_INSTANCE;
	Instance.contents.instance.master = input;
	Instance.contents.instance.view = connview;
	Instance.contents.instance.facet = "interface";
	Instance.contents.instance.version = OCT_CURRENT_VERSION;
	Instance.contents.instance.transform.transformType = OCT_NO_TRANSFORM;
	Instance.contents.instance.transform.translation.x = x;
	Instance.contents.instance.transform.translation.y = y;
  
  
	OH_ASSERT(octCreate(&Palette,&Instance));

	OH_ASSERT(octAttach(&ConnectorBag,&Instance));
	OH_ASSERT(octAttach(&InstanceBag,&Instance));

	OH_ASSERT(octBB(&Instance,&Box));
	x += Box.upperRight.x - Box.lowerLeft.x + padding;
	height = Box.upperRight.x - Box.lowerLeft.x;
	if ( height > rowHeight ) rowHeight = height;

	if (++i > 20 ) { 
	    i = 0; printf("\n");
	    y += padding + rowHeight;
	    x = 0;
	    rowHeight = 0;
	}
	printf("."); fflush(stdout);
    
    }
  
    printf("\nFinished %s\n", pname);

    OH_ASSERT(octFlushFacet(&Palette));
    {
	char command[1000];
	sprintf( command, "tapPP %s %s", pname, paletteview );
	printf( "Running tapPP: %s ...", command ); fflush( stdout );
	util_csystem( command );
    }

    sprintf(tname,"$OCTTOOLS/tech/%s/tap.views", technology);
    printf("\nUpdating %s\n", tname);
    OH_ASSERT(ohOpenFacet( &TapView, tname, "symbolic", "contents", "a" ));
    OH_ASSERT(ohGetOrCreateBag( &TapView, &bag, "PALETTES" ) );
    if ( ohGetByInstName( &bag, &TapInstance, "connector" ) == OCT_OK ) {
	OH_ASSERT(octDelete(&TapInstance));
	OH_ASSERT(octBB(&TapView,&Box));
    }
    TapInstance.contents.instance.master = pname;
    TapInstance.contents.instance.view = paletteview;
    TapInstance.contents.instance.facet = "interface";
    TapInstance.contents.instance.transform.transformType = OCT_NO_TRANSFORM;
    TapInstance.contents.instance.transform.translation.x = 0;
    TapInstance.contents.instance.transform.translation.y = Box.upperRight.y + 1000;
    OH_ASSERT(octCreate(&bag,&TapInstance));


    OH_ASSERT(octCloseFacet(&Palette));
    OH_ASSERT(octCloseFacet(&TapView));

    exit(0);
}







