#include "copyright.h"
#include "port.h"
#include "utility.h"
#include "oct.h"
#include "oh.h"
#include "th.h"
#include "errtrap.h"
#include "options.h" 

optionStruct optionList[] = {
    {"t", "tech name",	"Specify technology name (default to scmos)" },
    {"c", "contact name", "(normally this is determined automatically)" },
    {"d", 0,              "Debug flag"},
    { OPT_RARG, "cellType" , "Type of cell (1st_layer[-2nd_layer])" },
    { OPT_RARG, "width",     "width (in lambdas)" },
    { OPT_RARG, "height",     "height (in lambdas)" },
    {0,0,0}
};


/*
 *  Copyright (c) 1986 by Jeffrey L. Burns.  All rights reserved.
 *
 *  Main routine for making simple leaf cells of type physical.
 *
 *  4/3/86
 *  last modified 3/20/88
 *
 *  Modified 2/22/92 by Paul B. Cohen (paul@aha.com) to accomodate
 *  NWEL and PWEL
 *
 *  Modified 10/9/92 by Edoardo Charbon (charbon@ic.berkeley.edu) to 
 *  accomodate superconducting technology nbjj (UC-Berkeley)
 *
 */

main(argc, argv)
int argc;
char *argv[];
{
    int width;			/*  inputted width in lambda  */
    int height;			/*  inputted height in lambda  */
    int mx, mn;			/*  */
    double dwidth;		/*  inputted width in lambda  */
    double dheight;		/*  inputted height in lambda  */
    char cellName[80];		/*  buffer to hold cell's name  */
    char *cellNameArg;		/* From the command line arg. */
    char *fullCellName = 0;
    octObject Cell, prop;
    char *ptr;
    char *technology = "scmos";
    int debug = 0;
    
    int c;
    extern int optind;
    extern char *optarg;

    while ( (c=optGetOption( argc,argv ) ) != EOF ) {
	switch (c) {
	case 't': technology = util_strsav(optarg) ; break;
	case 'c': fullCellName = util_strsav(optarg); break;
	case 'd': debug = 1; break;
	default: optUsage();
	}
    }

    if ( argc - optind != 3 ) {
	optUsage();
    }

    cellNameArg = util_strsav( argv[optind++] );
    width = atoi(argv[optind++]);
    height = atoi(argv[optind++]);


    octBegin();

    /*
     *  Make the cell name.
     */
    mx = MAX(width, height);
    mn = MIN(width, height);

    if (strlen(argv[1]) >= 30)  {
	errRaise("prim", 1, "cell name %s is too long.\n", argv[1]);
    }


    if ( fullCellName ) {
	(void) sprintf(cellName, "%s", fullCellName );
    } else {
	(void) sprintf(cellName, "%s.%dx%d", cellNameArg, mx, mn);
    }

     /* Fill in the fields of the Cell that are constant. */
    Cell.type = OCT_FACET;
    Cell.contents.facet.cell = cellName;
    Cell.contents.facet.view = "physical";
    Cell.contents.facet.facet = "contents";
    Cell.contents.facet.version = OCT_CURRENT_VERSION;
    Cell.contents.facet.mode = "w";

    if ( octOpenFacet( &Cell ) < OCT_OK ) {
	errRaise( "prim", 1 , "Cannot open %s. \n\t%s", 
		 ohFormatName( &Cell ), octErrorString() );
    }
    ohCreatePropStr( &Cell, &prop, "TECHNOLOGY", technology );

    thTechnology( &Cell ); /* Let th know which technology we are using. */

    width *= thLambda();
    height *= thLambda();

    if (strcmp(technology, "scmos") == 0) {
	primScmosMapping(&Cell, cellNameArg, width, height);
    } else if (strcmp(technology, "super") == 0) {
	primSuperMapping(&Cell, cellNameArg, width, height);
    } else {
	(void) fprintf(stderr, "prim: Technology %s is not in our data-base.\n", 
		       technology);
    }	    
    
    octEnd();
    exit(0);
    
}


/*
 *  Mapps the name or the type of the cell with a physical structure.
 */
int
primScmosMapping(Cell, cellNameArg, width, height)
    octObject *Cell;
    char *cellNameArg;
    int width, height;
{
    char *ptr, *first, *second;
    char *conLayer;
    
    if ((ptr = strchr(cellNameArg, '-')) != 0) {
	first = cellNameArg;
	second = ptr + 1;
	*ptr = '\0';
	
	(void) fprintf(stderr, "generating a %s-%s (%dx%d) connector\n", 
		       first, second, height, width);
	
	if (strcmp(first, second) == 0) {
	    primOneLayerContact( Cell, first, width, height);
	} else {
	    /* XXX argh!!! */
	    if ((strcmp(first, "NDIF") == 0) || (strcmp(second, "NDIF") == 0)) {
		conLayer = "COND";
	    } else if ((strcmp(first, "PDIF") == 0) || 
		       (strcmp(second, "PDIF") == 0)) {
		conLayer = "COPD";
	    } else if ((strcmp(first, "POLY") == 0) || 
		       (strcmp(second, "POLY") == 0)) {
		conLayer = "COPO";
	    } else if ((strcmp(first, "MET2") == 0) || 
		       (strcmp(second, "MET2") == 0)) {
		conLayer = "COM2";
	    }
	    primThreeLayerContact(Cell, conLayer, first, second, width, height);
	}	
    } else if (strcmp(cellNameArg, "cond") == 0)  {
	primThreeLayerContact(Cell, "COND", "NDIF", "MET1", width, height);
    } else if (strcmp(cellNameArg, "copd") == 0)  {
	primThreeLayerContact(Cell, "COPD", "PDIF", "MET1", width, height);
    } else if (strcmp(cellNameArg, "copo") == 0)  {
	primThreeLayerContact(Cell, "COPO", "POLY", "MET1", width, height);
    } else if (strcmp(cellNameArg, "com2") == 0)  {
	primThreeLayerContact(Cell, "COM2", "MET1", "MET2", width, height);
    } else if (strcmp(cellNameArg, "m1m1") == 0)  {
	primOneLayerContact(Cell, "MET1", width, height);
    } else if (strcmp(cellNameArg, "m2m2") == 0)  {
	primOneLayerContact(Cell, "MET2", width, height);
    } else if (strcmp(cellNameArg, "ndnd") == 0)  {
	primOneLayerContact(Cell, "NDIF", width, height);
    } else if (strcmp(cellNameArg, "pdpd") == 0)  {
	primOneLayerContact(Cell, "PDIF", width, height);
    } else if (strcmp(cellNameArg, "popo") == 0)  {
	primOneLayerContact(Cell, "POLY", width, height);
    } else if (strcmp(cellNameArg, "pwpw") == 0)  {
	primOneLayerContact(Cell, "PWEL", width, height);
    } else if (strcmp(cellNameArg, "nwnw") == 0)  {
	primOneLayerContact(Cell, "NWEL", width, height);
    } else  {
	printf("cell type \"%s\" is not recognized; ", cellNameArg);
	printf("the following are legal.\n");
	printf("    3-layer contacts:  com2, copo, cond, copd\n");
	printf("    1-layer contacts:  m1m1, m2m2, ndnd, pdpd, popo, pwpw nwnw\n");
 	exit(-1);
    }

}



int
primSuperMapping(Cell, cellNameArg, width, height)
    octObject *Cell;
    char *cellNameArg;
    int width, height;
{
    char *ptr, *first, *second;
    char *conLayer, *conLayer2;
    
    if ((ptr = strchr(cellNameArg, '-')) != 0) {
	first = cellNameArg;
	second = ptr + 1;
	*ptr = '\0';
	
	(void) fprintf(stderr, "generating a %s-%s (%dx%d) connector\n", 
		       first, second, height, width);
	
	if (strcmp(first, second) == 0) {
	    primOneLayerContact( Cell, first, width, height);
	} else {
	    if ((strcmp(first, "MC") == 0) || 
		(strcmp(second, "MB") == 0)) {
		conLayer = "JC";
		primThreeLayerContact(Cell, conLayer, first, second, 
				      width, height);
	    } else if ((strcmp(first, "MB") == 0) || 
		(strcmp(second, "MC") == 0)) {
		conLayer = "JC";
		primThreeLayerContact(Cell, conLayer, first, second, 
				      width, height);
	    } else if ((strcmp(first, "MB") == 0) || 
		       (strcmp(second, "MG") == 0)) {
		conLayer = "GC";
		conLayer2 = "RC";
		primFourLayerContact(Cell, conLayer, conLayer2,
				     first, second, width, height);
	    } else if ((strcmp(first, "MC") == 0) || 
		       (strcmp(second, "MG") == 0)) {
		/* three inter layers needed */
		fprintf(stderr, "Fatal Error: not implemented yet\n");
		exit(-1);
	    } else if ((strcmp(first, "MR") == 0) || 
		       (strcmp(second, "MG") == 0)) {
		/* three inter layers needed */
		fprintf(stderr, "Fatal Error: not implemented yet\n");
		exit(-1);
	    } else if ((strcmp(first, "MR") == 0) || 
		       (strcmp(second, "MB") == 0)) {
		conLayer = "RC";
		primThreeLayerContact(Cell, conLayer, first, second, 
				      width, height);
	    }
	}	

	/* short names alias */

    } else if (strcmp(cellNameArg, "mbmc") == 0)  {
	primThreeLayerContact(Cell, "JC", "MB", "MC", width, height);
    } else if (strcmp(cellNameArg, "mbgnd") == 0)  {
	primFourLayerContact(Cell, "GC", "RC", "MB", "MG", width, height);
    } else if (strcmp(cellNameArg, "mcgnd") == 0)  {
	fprintf(stderr, "Fatal Error: not implemented yet\n");
		exit(-1);
    } else if (strcmp(cellNameArg, "mrgnd") == 0)  {
	fprintf(stderr, "Fatal Error: not implemented yet\n");
	exit(-1);
    } else if (strcmp(cellNameArg, "mrmb") == 0)  {
	primThreeLayerContact(Cell, "RC", "MR", "MB", width, height);
    } else if (strcmp(cellNameArg, "mbmb") == 0)  {
	primOneLayerContact(Cell, "MB", width, height);
    } else if (strcmp(cellNameArg, "mcmc") == 0)  {
	primOneLayerContact(Cell, "MC", width, height);
    } else if (strcmp(cellNameArg, "mrmr") == 0)  {
	primOneLayerContact(Cell, "MR", width, height);
    } else if (strcmp(cellNameArg, "gnd") == 0)  {
	primOneLayerContact(Cell, "MG", width, height);
    } else {
	printf("cell type \"%s\" is not recognized; ", cellNameArg);
	printf("the following are legal.\n");
	printf("    3/4-layer contacts:  mcmb, mrmb, mbgnd, mcgnd, mrgnd\n");
	printf("    1-layer contacts:  mbmb, mcmc, mrmr gnd\n");
 	exit(-1);
    }
}
