#include "copyright.h"
#include "port.h"
#include "utility.h"
#include "errtrap.h"
#include "oct.h"
#include "oh.h"

/*LINTLIBRARY*/


static void addImplBoxToConn( topBag, implBoxPtr )
    octObject *topBag;		
    octObject *implBoxPtr;	/* implementing box on connector terminal */
{
    octObject layer;
    octObject prop;
    octObject layerBag;
    struct octPoint *llPtr = &implBoxPtr->contents.box.lowerLeft;
    struct octPoint *urPtr = &implBoxPtr->contents.box.upperRight;
    int width, height;


    
    width = MIN( -llPtr->x, urPtr->x ) * 2;
    height = MIN( -llPtr->y, urPtr->y ) * 2;

    prop.type = OCT_PROP;
    prop.contents.prop.name = "WIDTHS";
    prop.contents.prop.type = OCT_INTEGER_ARRAY;
    prop.contents.prop.value.integer_array.length = 4;
    prop.contents.prop.value.integer_array.array = ALLOC( octCoord, 4 );
    prop.contents.prop.value.integer_array.array[0] = height;
    prop.contents.prop.value.integer_array.array[1] = width;
    prop.contents.prop.value.integer_array.array[2] = height;
    prop.contents.prop.value.integer_array.array[3] = width;

    OH_ASSERT( octGenFirstContainer(implBoxPtr, OCT_LAYER_MASK, &layer) );
    OH_ASSERT( ohCreateBag( topBag, &layerBag, layer.contents.layer.name ) );
    OH_ASSERT( octCreate( &layerBag, &prop ) );

}



static void doConn( facet, inst )
    octObject *facet;
    octObject *inst;
{
    octObject master;
    octGenerator implGen;
    struct octBox bBox;
    octObject implGeo;
    octStatus genStatus;
    octObject term;
    octObject bigBag;
    struct connLayer *layerDescs;
    double area;
    int    actualLayers;
    int i;
    octObject prop;
    octObject topBag;

    
    OH_ASSERT( ohOpenMaster(&master, inst, "contents", "r") );

    OH_ASSERT(octGenFirstContent(&master, OCT_TERM_MASK, &term)); /* Must have at least a term */
    OH_ASSERT(octBB(&master, &bBox)); /* must have a bounding box. */


    area = (double) (bBox.upperRight.x - bBox.lowerLeft.x) *
		    (double) (bBox.upperRight.y - bBox.lowerLeft.y);

    actualLayers = ohCountContents( &master, OCT_LAYER_MASK );

    ohGetOrCreateBag( facet, &bigBag, "SORTED_CONNECTORS" );
    ohCreateBag( &bigBag, &topBag, inst->contents.instance.master );

    /* examine terminal implementation geometries */
    OH_ASSERT(octInitGenContents(&term, OCT_BOX_MASK, &implGen));
    while ((genStatus = octGenerate(&implGen, &implGeo)) == OCT_OK) {
	addImplBoxToConn( &topBag, &implGeo );
    }

    OH_ASSERT(octCloseFacet(&master));


    ohCreatePropInt( &topBag, &prop, "AREA", area );
    ohCreatePropInt( &topBag, &prop, "ACTUALLAYERS", actualLayers );
    ohCreatePropStr( &topBag, &prop, "INSTANCEMASTER", inst->contents.instance.master );
    ohCreatePropStr( &topBag, &prop, "INSTANCEVIEW", inst->contents.instance.view );
    ohCreatePropStr( &topBag, &prop, "INSTANCEFACET", inst->contents.instance.facet );
    ohCreatePropStr( &topBag, &prop, "INSTANCEVERSION", inst->contents.instance.version );
    
}



main( argc, argv )
    int argc;
    char** argv;
{
    char* cell = 0;
    char* view = "symbolic";
    octObject cfacet, ifacet;
    octObject prop;
    octGenerator genInst;
    octObject inst, box, layer, label;


    errProgramName( "tapPP" );

    if ( argc > 2 ) {
	view = argv[2];
    }
    if ( argc < 2 ) {
	printf( "usage: tapPP <palette> [view]\n" );
	exit( 0 );
    }
    cell = argv[1];


    OH_ASSERT(ohOpenFacet( &ifacet, cell, view, "interface", "w" ));
    OH_ASSERT(ohOpenFacet( &cfacet, cell, view, "contents", "r" ));

    OH_ASSERT(ohGetByPropName( &cfacet, &prop, "TECHNOLOGY" ));
    octCreate( &ifacet, &prop );
    OH_ASSERT(ohGetByPropName( &cfacet, &prop, "VIEWTYPE" ));
    octCreate( &ifacet, &prop );
    OH_ASSERT(ohGetByPropName( &cfacet, &prop, "EDITSTYLE" ));
    octCreate( &ifacet, &prop );
    ohCreateBox( &ifacet, &box, 0, 0, 100 , 100 ); /* Give a box to the facet */

    ohCreateLayer( &ifacet, &layer, "POLY" );

    label.type = OCT_LABEL;
    label.contents.label.label = "This is the FAST connector facet\ngenerated by tapPP.";
    label.contents.label.region.lowerLeft.x = 0;
    label.contents.label.region.lowerLeft.y = 0;
    label.contents.label.region.upperRight.x = 1000;
    label.contents.label.region.upperRight.y = 500;
    label.contents.label.textHeight = 200;
    label.contents.label.vertJust = OCT_JUST_CENTER;
    label.contents.label.horizJust = OCT_JUST_CENTER;
    label.contents.label.lineJust = OCT_JUST_CENTER;
    
    OH_ASSERT( octCreate(  &layer, &label ) );

    octInitGenContents( &cfacet, OCT_INSTANCE_MASK, &genInst );
    while( octGenerate( &genInst, &inst ) == OCT_OK ) {
	doConn( &ifacet, &inst );
    }
    octFlushFacet( &ifacet );
    exit( 1 );
}
