/*
*	Support the ability to undo additions to the facet if we find
*  something wrong down the line.  Call the function addToBackout() to
*  put an object on the backout list, the function clearBackout() when
*  the changes are known good, and the function backout() to delete all
*  the objects on the backout list.
*/
#include "tech-edit.h"
#include "utility.h"

struct backoutEntry {
    octId id;
    struct backoutEntry *next;
};

static struct backoutEntry *backoutList = NIL(struct backoutEntry);

addToBackout(objPtr)
octObject *objPtr;
{
    struct backoutEntry *entryPtr;

    entryPtr = ALLOC(struct backoutEntry, 1);
    entryPtr->id = objPtr->objectId;
    entryPtr->next = backoutList;
    backoutList = entryPtr;
}

clearBackout()
{
    struct backoutEntry *entryPtr;

    while (backoutList) {
	entryPtr = backoutList;
	backoutList = backoutList->next;
	FREE(entryPtr);
    }
}

octStatus backout()
{
    struct backoutEntry *entryPtr;
    octObject obj;

    while (backoutList) {
	entryPtr = backoutList;
	backoutList = backoutList->next;
	obj.objectId = entryPtr->id;
	FREE(entryPtr);
	OCT_ASSERT(octGetById(&obj));
	OCT_ASSERT(octDelete(&obj));
    }
}
