#include "tech-edit.h"
#include "utility.h"

/*ARGSUSED*/
finish(spotPtr, cmdList, userOptionWord)
RPCSpot *spotPtr;
lsList cmdList;
long userOptionWord;
/*
 * Terminates the application.
 */
{
    VEM_ASSERT(vemFreeSelSet(maxSelSet));
    VEM_ASSERT(vemFreeSelSet(minSelSet));
    RPCExit(0);
}

/*
 * Command for creating a minimum rule.
 * Format for the command:  points(2) : min-rule.
 */

/*ARGSUSED*/
createMinRule(spotPtr, cmdList, userOptionWord)
RPCSpot *spotPtr;
lsList cmdList;
long userOptionWord;
{
    RPCArg *firstArgPtr;

    ERROR_SET();

    checkTechProp();

    if (lsLength(cmdList) != 1) {
	VEMMSG("\nformat:  points(2) : min-rule\n");
	vemPrompt();
	return(RPC_OK);
    }

    lsFirstItem(cmdList, (Pointer *) &firstArgPtr, NIL(lsHandle));
    if (firstArgPtr->argType != VEM_POINT_ARG || firstArgPtr->length != 2) {
	VEMMSG("\nformat:  points(2) : min-rule\n");
	vemPrompt();
	return(RPC_OK);
    }

    return(doCreateRule(firstArgPtr, MIN_RULE));
}

/*
 * Command for creating a maximum rule.
 * Format for the command:  points(2) : max-rule.
 */

/*ARGSUSED*/
createMaxRule(spotPtr, cmdList, userOptionWord)
RPCSpot *spotPtr;
lsList cmdList;
long userOptionWord;
{
    RPCArg *firstArgPtr;

    ERROR_SET();

    checkTechProp();

    if (lsLength(cmdList) != 1) {
	VEMMSG("\nformat:  points(2) : max-rule\n");
	vemPrompt();
	return(RPC_OK);
    }

    lsFirstItem(cmdList, (Pointer *) &firstArgPtr, NIL(lsHandle));
    if (firstArgPtr->argType != VEM_POINT_ARG || firstArgPtr->length != 2) {
	VEMMSG("\nformat:  points(2) : max-rule\n");
	vemPrompt();
	return(RPC_OK);
    }

    return(doCreateRule(firstArgPtr, MAX_RULE));
}

doCreateRule(argPtr, ruleType)
RPCArg *argPtr;
int ruleType;
{
    octObject edge1, edge2;
    octObject bigBag, ruleBag;

    if (argPtr->argData.points[0].facet != theFacet.objectId ||
		    argPtr->argData.points[1].facet != theFacet.objectId) {
	ERROR("\nPoints must be in facet being edited\n");
    }

    convertPointToEdge(&argPtr->argData.points[0], &edge1);
    convertPointToEdge(&argPtr->argData.points[1], &edge2);

    bigBag.type = OCT_BAG;
    bigBag.contents.bag.name = BIG_BAG_NAME;
    OCT_ASSERT(octGetOrCreate(&theFacet, &bigBag));

    ruleBag.type = OCT_BAG;
    ruleBag.contents.bag.name = (ruleType == MAX_RULE) ? MAX_NAME : MIN_NAME;
    OCT_ASSERT(octCreate(&bigBag, &ruleBag));

    OCT_ASSERT(octAttach(&ruleBag, &edge1));
    OCT_ASSERT(octAttach(&ruleBag, &edge2));

    makeArrow(&ruleBag, &argPtr->argData.points[0].thePoint,
			&argPtr->argData.points[1].thePoint);

    VEM_ASSERT(vemAddSelSet((ruleType == MAX_RULE) ? maxSelSet : minSelSet,
		    ruleBag.objectId));

    clearBackout();
    return(RPC_OK);
}

makeArrow(bagPtr, end1Ptr, end2Ptr)
octObject *bagPtr;
struct octPoint *end1Ptr, *end2Ptr;
{
    octObject arrowPath;
    struct octPoint pointList[8];
    struct octPoint parVector, perpVector;
    double scale;
    double sqrt();

    parVector.x = end2Ptr->x - end1Ptr->x;
    parVector.y = end2Ptr->y - end1Ptr->y;
    scale = parVector.x * parVector.x + parVector.y * parVector.y;
    scale = sqrt(scale) / LAMBDA;
    if (scale < 4.0) scale = 4.0;
    parVector.x = parVector.x / scale;
    parVector.y = parVector.y / scale;

    perpVector.x = parVector.y / 2;
    perpVector.y = -parVector.x / 2;

    pointList[0].x = end1Ptr->x + parVector.x - perpVector.x;
    pointList[0].y = end1Ptr->y + parVector.y - perpVector.y;

    pointList[1] = *end1Ptr;

    pointList[2].x = end1Ptr->x + parVector.x + perpVector.x;
    pointList[2].y = end1Ptr->y + parVector.y + perpVector.y;

    pointList[3] = *end1Ptr;

    pointList[4] = *end2Ptr;

    pointList[5].x = end2Ptr->x - parVector.x - perpVector.x;
    pointList[5].y = end2Ptr->y - parVector.y - perpVector.y;

    pointList[6] = *end2Ptr;

    pointList[7].x = end2Ptr->x - parVector.x + perpVector.x;
    pointList[7].y = end2Ptr->y - parVector.y + perpVector.y;

    arrowPath.type = OCT_PATH;
    arrowPath.contents.path.width = 0;
    OCT_ASSERT(octCreate(bagPtr, &arrowPath));
    OCT_ASSERT(octPutPoints(&arrowPath, 8, pointList));
}

/*
 * Command for displaying a textual description of a rule
 * Format for the command:  : show-rule (over the object).
 */

/*ARGSUSED*/
showRule(spotPtr, cmdList, userOptionWord)
RPCSpot *spotPtr;
lsList cmdList;
long userOptionWord;
{
    int numRules;
    dmWhichItem *ruleList;
    int i;

    ERROR_SET();

    checkTechProp();

    numRules = findRules(spotPtr, cmdList, &ruleList);

    if (numRules != 1) {
	char buf[100];

	(void) sprintf(buf, "\n   %d rules", numRules);
	BOLD(buf);
    }

    for (i = 0; i < numRules; i++) {
	VEMMSG("\n");
	BOLD(ruleList[i].itemName);
    }
    VEMMSG("\n");

    freeRuleList(ruleList, numRules);

    vemPrompt();
    return(RPC_OK);
}

/*
 * Command for deleting a rule.
 * Format for the command:  : delete-rule (over the object).
 */

/*ARGSUSED*/
deleteRule(spotPtr, cmdList, userOptionWord)
RPCSpot *spotPtr;
lsList cmdList;
long userOptionWord;
{
    int numRules;
    dmWhichItem *ruleList;
    int i;

    ERROR_SET();

    checkTechProp();

    numRules = findRules(spotPtr, cmdList, &ruleList);

    if (numRules == 1) {
	doDeleteRule((octId) ruleList[0].userData);
    } else if (dmMultiWhich("Rule Deletion List",
			numRules, ruleList, (int (*)()) 0) == VEM_OK) {
	for (i = 0; i < numRules; i++) {
	    if (ruleList[i].flag) {
		doDeleteRule((octId) ruleList[i].userData);
	    }
	}
    }

    freeRuleList(ruleList, numRules);

    return(RPC_OK);
}

doDeleteRule(ruleBagId)
octId ruleBagId;
{
    octObject ruleBag, obj;
    octGenerator gen;
    octStatus status;

    ruleBag.objectId = ruleBagId;
    OCT_ASSERT(octGetById(&ruleBag));

    if (strcmp(ruleBag.contents.bag.name, MIN_NAME) == 0) {
	VEM_ASSERT(vemDelSelSet(minSelSet, ruleBag.objectId));
    } else if (strcmp(ruleBag.contents.bag.name, MAX_NAME) == 0) {
	VEM_ASSERT(vemDelSelSet(maxSelSet, ruleBag.objectId));
    }

    OCT_ASSERT(octInitGenContents(&ruleBag,OCT_ALL_MASK,&gen));
    while ((status = octGenerate(&gen, &obj)) == OCT_OK) {
	OCT_ASSERT(octDelete(&obj));
    }
    OCT_ASSERT(status);

    OCT_ASSERT(octDelete(&ruleBag));
}
