#include "tech-edit.h"

convertPointToEdge(pointPtr, edgePtr)
RPCPointArg *pointPtr;
octObject *edgePtr;
{
    octObject layer, geo;
    octGenerator layerGen, geoGen;
    octStatus status;
    int numFound = 0;

    OCT_ASSERT(octInitGenContents(&theFacet, OCT_LAYER_MASK, &layerGen));
    while ((status = octGenerate(&layerGen, &layer)) == OCT_OK) {
	/* XXX should handle things other than boxes */
	OCT_ASSERT(octInitGenContents(&layer, OCT_BOX_MASK, &geoGen));
	while ((status = octGenerate(&geoGen, &geo)) == OCT_OK) {
	    numFound += matchPointToBox(pointPtr, &geo, edgePtr, &layer);
	}
	OCT_ASSERT(status);
    }
    OCT_ASSERT(status);

    switch (numFound) {
	case 0:
	    ERROR("\nPoint not on an edge\n");
	case 1:
	    return;
	default:
	    ERROR("\nAmbiguous point\n");
    }
}

matchPointToBox(pointPtr, boxPtr, edgePtr, layerPtr)
RPCPointArg *pointPtr;
octObject *boxPtr;
octObject *edgePtr;
octObject *layerPtr;
{
    /* check the left edge */
    if (pointPtr->thePoint.x == boxPtr->contents.box.lowerLeft.x) {
	if (pointPtr->thePoint.y == boxPtr->contents.box.lowerLeft.y ||
		pointPtr->thePoint.y == boxPtr->contents.box.upperRight.y) {
	    return(2);
	}
	if (pointPtr->thePoint.y < boxPtr->contents.box.lowerLeft.y ||
		pointPtr->thePoint.y > boxPtr->contents.box.upperRight.y) {
	    return(0);
	}

	edgePtr->type = OCT_EDGE;
	edgePtr->contents.edge.start.x = pointPtr->thePoint.x;
	edgePtr->contents.edge.start.y = boxPtr->contents.box.upperRight.y;
	edgePtr->contents.edge.end.x = pointPtr->thePoint.x;
	edgePtr->contents.edge.end.y = boxPtr->contents.box.lowerLeft.y;
	OCT_ASSERT(octCreate(layerPtr, edgePtr));
	addToBackout(edgePtr);
	return(1);
    }

    /* check the right edge */
    if (pointPtr->thePoint.x == boxPtr->contents.box.upperRight.x) {
	if (pointPtr->thePoint.y == boxPtr->contents.box.lowerLeft.y ||
		pointPtr->thePoint.y == boxPtr->contents.box.upperRight.y) {
	    return(2);
	}
	if (pointPtr->thePoint.y < boxPtr->contents.box.lowerLeft.y ||
		pointPtr->thePoint.y > boxPtr->contents.box.upperRight.y) {
	    return(0);
	}

	edgePtr->type = OCT_EDGE;
	edgePtr->contents.edge.start.x = pointPtr->thePoint.x;
	edgePtr->contents.edge.start.y = boxPtr->contents.box.lowerLeft.y;
	edgePtr->contents.edge.end.x = pointPtr->thePoint.x;
	edgePtr->contents.edge.end.y = boxPtr->contents.box.upperRight.y;
	OCT_ASSERT(octCreate(layerPtr, edgePtr));
	addToBackout(edgePtr);
	return(1);
    }

    /* check the bottom edge */
    if (pointPtr->thePoint.y == boxPtr->contents.box.lowerLeft.y) {
	if (pointPtr->thePoint.x < boxPtr->contents.box.lowerLeft.x ||
		pointPtr->thePoint.x > boxPtr->contents.box.upperRight.x) {
	    return(0);
	}

	edgePtr->type = OCT_EDGE;
	edgePtr->contents.edge.start.x = boxPtr->contents.box.lowerLeft.y;
	edgePtr->contents.edge.start.y = pointPtr->thePoint.y;
	edgePtr->contents.edge.end.x = boxPtr->contents.box.upperRight.y;
	edgePtr->contents.edge.end.y = pointPtr->thePoint.y;
	OCT_ASSERT(octCreate(layerPtr, edgePtr));
	addToBackout(edgePtr);
	return(1);
    }

    /* check the top edge */
    if (pointPtr->thePoint.y == boxPtr->contents.box.upperRight.y) {
	if (pointPtr->thePoint.x < boxPtr->contents.box.lowerLeft.x ||
		pointPtr->thePoint.x > boxPtr->contents.box.upperRight.x) {
	    return(0);
	}

	edgePtr->type = OCT_EDGE;
	edgePtr->contents.edge.start.x = boxPtr->contents.box.upperRight.y;
	edgePtr->contents.edge.start.y = pointPtr->thePoint.y;
	edgePtr->contents.edge.end.x = boxPtr->contents.box.lowerLeft.y;
	edgePtr->contents.edge.end.y = pointPtr->thePoint.y;
	OCT_ASSERT(octCreate(layerPtr, edgePtr));
	addToBackout(edgePtr);
	return(1);
    }

    /* wasn't on any edge */
    return(0);
}
