#include <sys/types.h>
#include "tech-edit.h"
#include "tap.h"

#ifdef OLD_X10_CODE
struct bitmapEdit {
    Window window;		/* parent window of bitmap editing redion */
    XAssocTable *bitIndexTable;	/* find bit position from subwindow id */
    int width, height;		/* dimensions of bitmap */
    char *bitString;		/* contains '1' and '0' entries */
};

struct layerLooks {
    struct layerLooks *next;			/* free list link */
    Window dWin;				/* dialog box for this layer */
    int bgPixel;				/* background color of dialog */

    octId layerId;				/* oct id of layer */
    char *deviceName;				/* name of display device */
    int titleField;				/* field id of title */
    int priority;				/* display priority of layer */
    int priorityField;				/* field id of priority */

    int nColors;				/* number of colors */
    int currentColorIndex;			/* color being displayed now */
    int nColorsField, colorIndexField;		/* field ids for color counts */
    Window colorIndRow, sliderRow;		/* X window ids of color area */
    Color *colorVals;				/* RGB values of all colors */
    Window redSlide, greenSlide, blueSlide;	/* X window ids of sliders */

    int fillSize, borderSize;			/* size of patterns */
    int fillField, borderField;			/* field ids of pattern sizes */
    struct bitmapEdit fillBitmap, borderBitmap;	/* pattern edit windows */

    Window patchWindow;				/* window for stippled patch */

    int tryField, OKField, abortField;		/* field ids for buttons */
};

static Color redPixel, greenPixel, bluePixel;	/* colors of RGB slide bars */

static struct layerLooks *freeList = 0;		/* free list of structures */

    /* display device whose display attributes are being edited */
static char *dispTypeName;
static int dispTypeChromatism;

    /* find layerLooks structure from slide bars and bitmap edit windows */
static XAssocTable *looksTable;

extern double sqrt();

/*ARGSUSED*/
setDevice(spotPtr, cmdList, userOptionWord)
RPCSpot *spotPtr;
lsList cmdList;
long userOptionWord;
{
    int numDevices;
    dmWhichItem *deviceList;
    int chosen;
    static dmTextItem nameItem = { "", 0, 30, "", 0 };

    ERROR_SET();

    checkTechProp();

    if (lsLength(cmdList) != 0) {
	USAGE("\nformat:  : display-device\n");
    }

    numDevices = findDevices(&deviceList);
    if (dmWhichOne("Display Device Type Name", numDevices, deviceList,
					&chosen, (int (*)()) 0) == VEM_OK) {
	if (chosen == numDevices - 1) {
	    nameItem.value = "";
	    if (dmMultiText("Enter Device Name", 1, &nameItem) == VEM_OK) {
		dispTypeName = util_strsav(nameItem.value);
		dispTypeChromatism =
		    (dmConfirm("Color Device?", "", "Yes", "No") == VEM_OK) ?
			TAP_COLOR : TAP_BW;
	    }
	} else {
	    dispTypeName = util_strsav(deviceList[chosen].itemName);
	    dispTypeChromatism = (int) deviceList[chosen].userData;
	}

	vemSetDisplayType(dispTypeName, dispTypeChromatism);
	TAP_ASSERT(tapSetDisplayType(dispTypeName, dispTypeChromatism));
    }
    FREE(deviceList);

    return(RPC_OK);
}

/*ARGSUSED*/
editLayer(spotPtr, cmdList, userOptionWord)
RPCSpot *spotPtr;
lsList cmdList;
long userOptionWord;
{
    RPCArg *argPtr;
    octObject layer;
    char layerName[1000];		/* hopefully enough */
    struct layerLooks *buildLooksEditor();
    void handleEvent();

    ERROR_SET();

    checkTechProp();

    /* get/create the layer */
    layer.type = OCT_LAYER;
    switch (lsLength(cmdList)) {
	case 0:
	    if (vuFindLayer(&theFacet, &(spotPtr->thePoint), layerName)
							    != VEM_OK) {
		USAGE("\nformat:  [layer-name] : layer-looks\n");
	    }
	    layer.contents.layer.name = layerName;
	    break;
	case 1:
	    LS_ASSERT(lsFirstItem(cmdList, (Pointer *) &argPtr, NIL(lsHandle)));
	    if (argPtr->argType != VEM_TEXT_ARG) {
		USAGE("\nformat:  [layer-name] : layer-looks\n");
	    }
	    layer.contents.layer.name = argPtr->argData.string;
	    break;
	default:
	    USAGE("\nformat:  [layer-name] : layer-looks\n");
    }
    OCT_ASSERT(octGetOrCreate(&theFacet, &layer));

    convertLayerToLooks(&layer, buildLooksEditor());

    /* get the initial exposure events */
    handleEvent();

    return(RPC_OK);
}

initLooksEditor()
{
    long readMask;
    long zero = 0;
    void handleEvent();

    vemGetDisplayType(&dispTypeName, &dispTypeChromatism);
    TAP_ASSERT(tapSetDisplayType(dispTypeName, dispTypeChromatism));

    XDM_ASSERT(XDMInit("tech-edit"));
    looksTable = XCreateAssocTable(32);
    if (looksTable == NIL(XAssocTable)) {
	ERROR("Couldn't allocate X association table");
    }

    XParseColor("red", &redPixel);
    XParseColor("green", &greenPixel);
    XParseColor("blue", &bluePixel);
    if (	!XGetHardwareColor(&redPixel) ||
		!XGetHardwareColor(&greenPixel) ||
		!XGetHardwareColor(&bluePixel)) {
	ERROR("Couldn't get enough colors for layer editing");
    }

    readMask = 1 << dpyno();

    RPCUserIO(dpyno()+1, (int *) &readMask, (int *) &zero, (int *) &zero,
	    handleEvent);
}

static struct layerLooks *buildLooksEditor()
{
    int bigColumn;
    int pMask;
    struct layerLooks *looksPtr;

    if (freeList) {
	looksPtr = freeList;
	freeList = freeList->next;
	XDM_ASSERT(XDMModify(looksPtr->dWin, looksPtr->tryField,
					XDM_VALUE, 0, XDM_END));
	XDM_ASSERT(XDMModify(looksPtr->dWin, looksPtr->OKField,
					XDM_VALUE, 0, XDM_END));
	XDM_ASSERT(XDMModify(looksPtr->dWin, looksPtr->abortField,
					XDM_VALUE, 0, XDM_END));
	return(looksPtr);
    }

    looksPtr = ALLOC(struct layerLooks);
    looksPtr->colorVals = ALLOC(Color);

    if ( ! XGetColorCells(0, 1, 0, &pMask, &(looksPtr->colorVals[0].pixel))) {
	/* must be B&W */
	looksPtr->colorVals[0].pixel = BlackPixel;
    }

    looksPtr->dWin = XDMDialogCreate(RootWindow);
    XMakeAssoc(looksTable, (XId) looksPtr->dWin, (caddr_t) looksPtr);
    XDM_ASSERT(XDMQuery(looksPtr->dWin, XDM_WINDOW,
				XDM_BGPIXEL, &(looksPtr->bgPixel), XDM_END));

    /* the column that contains everything */
    bigColumn = XDMRowColCreate(looksPtr->dWin, XDM_WINDOW);
    XDM_ASSERT(XDMModify(looksPtr->dWin, bigColumn, XDM_SPACE, 5, XDM_END));

    looksPtr->titleField = XDMTextCreate(looksPtr->dWin, bigColumn);

    buildEditField(looksPtr, bigColumn);

    buildPatchField(looksPtr, bigColumn);

    buildButtonField(looksPtr, bigColumn);

    return(looksPtr);
}

static convertLayerToLooks(layerPtr, looksPtr)
octObject *layerPtr;
struct layerLooks *looksPtr;
{
    int fillFlag, borderFlag, i;
    char *bits;
    char windowName[100];

    looksPtr->layerId = layerPtr->objectId;
    looksPtr->deviceName = util_strsav(dispTypeName);

    /* dialog box title */
    (void) sprintf(windowName, "Display of Layer %.20s on a %.30s",
				layerPtr->contents.layer.name,
				dispTypeName);
    XDM_ASSERT(XDMModify(looksPtr->dWin, looksPtr->titleField,
			    XDM_TEXT, windowName, XDM_END));

    /* actual display attributes */
    if (tapGetDisplayInfo(layerPtr, &(looksPtr->priority), &(looksPtr->nColors),
			    &fillFlag, &borderFlag) == OCT_OK) {

	    /* we can use tap to get the old values */
	if (looksPtr->nColors > 0) {
	    looksPtr->colorVals = REALLOCn(looksPtr->nColors, Color,
						    looksPtr->colorVals);
	    for (i = 0; i < looksPtr->nColors; i++) {
		looksPtr->colorVals[i].pixel = looksPtr->colorVals[0].pixel;
		TAP_ASSERT(tapGetDisplayColor(layerPtr, i,
					    &(looksPtr->colorVals[i].red),
					    &(looksPtr->colorVals[i].green),
					    &(looksPtr->colorVals[i].blue)));
	    }
	} else {
	    looksPtr->colorVals = REALLOCn(1, Color, looksPtr->colorVals);
	}

	if (fillFlag == TAP_EMPTY) {
	    looksPtr->fillBitmap.width = 0;
	    looksPtr->fillBitmap.height = 0;
	    bits = "";
	} else {
	    TAP_ASSERT(tapGetDisplayPattern(layerPtr, TAP_FILL_PATTERN,
					&(looksPtr->fillBitmap.width),
					&(looksPtr->fillBitmap.height),
					&bits));
	}
	looksPtr->fillBitmap.bitString = util_strsav(bits);

	if (borderFlag == TAP_EMPTY) {
	    looksPtr->borderBitmap.width = 0;
	    looksPtr->borderBitmap.height = 1;
	    bits = "";
	} else {
	    TAP_ASSERT(tapGetDisplayPattern(layerPtr, TAP_BORDER_PATTERN,
					&(looksPtr->borderBitmap.width),
					&(looksPtr->borderBitmap.height),
					&bits));
	}
	looksPtr->borderBitmap.bitString = util_strsav(bits);
    } else {

	    /* make up completely default values */
	looksPtr->priority = 0;

	looksPtr->colorVals = REALLOCn(1, Color, looksPtr->colorVals);
	if (dispTypeChromatism == TAP_COLOR) {
	    looksPtr->nColors = 1;
	    looksPtr->colorVals[0].red = 0;
	    looksPtr->colorVals[0].green = 0;
	    looksPtr->colorVals[0].blue = 0;
	} else {
	    looksPtr->nColors = 0;
	}

	looksPtr->fillBitmap.width = 1;
	looksPtr->fillBitmap.height = 1;
	looksPtr->fillBitmap.bitString = util_strsav("1");

	looksPtr->borderBitmap.width = 0;
	looksPtr->borderBitmap.height = 1;
	looksPtr->borderBitmap.bitString = util_strsav("");
    }

    /* layer drawing priority */
    drawEdRegField(looksPtr, looksPtr->priorityField);

    /* color count and index */
    drawEdRegField(looksPtr, looksPtr->nColorsField);
    looksPtr->currentColorIndex = 0;
    drawEdRegField(looksPtr, looksPtr->colorIndexField);

    /* fill bitmap */
    looksPtr->fillSize = looksPtr->fillBitmap.width;
    drawEdRegField(looksPtr, looksPtr->fillField);

    /* border bitmap */
    looksPtr->borderSize = looksPtr->borderBitmap.width;
    drawEdRegField(looksPtr, looksPtr->borderField);

    (void) sprintf(windowName, "layer-looks(%.70s)",
					    layerPtr->contents.layer.name);
    XDM_ASSERT(XDMModify(looksPtr->dWin, XDM_WINDOW,
					XDM_NAME, windowName, XDM_END));
    XDM_ASSERT(XDMPost(looksPtr->dWin, 0, 0, (int (*)()) 0, XDM_INTERACT));
}

static convertLooksToLayer(looksPtr)
struct layerLooks *looksPtr;
{
    octObject layer;			/* layer object */
    int i;

    /* get the layer */
    layer.objectId = looksPtr->layerId;
    if (octGetById(&layer) != OCT_OK) {
	ERROR("\nLayer has disappeared\n");
    }

    /* make sure we have up-to-date info */
    updateEdRegField(looksPtr, looksPtr->priorityField);
    updateEdRegField(looksPtr, looksPtr->nColorsField);

    /* set overall info */
    TAP_ASSERT(tapSetDisplayInfo(&layer, looksPtr->priority, looksPtr->nColors,
					TAP_STIPPLED, TAP_STIPPLED));

    /* colors */
    for (i = 0; i < looksPtr->nColors; i++) {
	TAP_ASSERT(tapSetDisplayColor(&layer, i,
			    looksPtr->colorVals[i].red,
			    looksPtr->colorVals[i].green,
			    looksPtr->colorVals[i].blue));
    }

    /* fill bitmap */
    TAP_ASSERT(tapSetDisplayPattern(&layer, TAP_FILL_PATTERN, 
			looksPtr->fillBitmap.width,
			looksPtr->fillBitmap.height,
			looksPtr->fillBitmap.bitString));

    /* border bitmap */
    TAP_ASSERT(tapSetDisplayPattern(&layer, TAP_BORDER_PATTERN, 
			looksPtr->borderBitmap.width,
			looksPtr->borderBitmap.height,
			looksPtr->borderBitmap.bitString));

    vemFlushTechInfo(&theFacet);
}

static destroyLooksEditor(looksPtr)
struct layerLooks *looksPtr;
{
    XDM_ASSERT(XDMEnd(looksPtr->dWin));

    XDestroySubwindows(looksPtr->fillBitmap.window);
    XDestroySubwindows(looksPtr->borderBitmap.window);

    FREE(looksPtr->deviceName);
    /* keep the pixel value */
    looksPtr->colorVals = REALLOCn(1, Color, looksPtr->colorVals);
    FREE(looksPtr->fillBitmap.bitString);
    FREE(looksPtr->borderBitmap.bitString);

    looksPtr->next = freeList;
    freeList = looksPtr;
}

static buildEditField(looksPtr, bigColumn)
struct layerLooks *looksPtr;
int bigColumn;
{
    int editField;
    int rowField;

    editField = XDMRowColCreate(looksPtr->dWin, bigColumn);
    XDM_ASSERT(XDMModify(looksPtr->dWin, editField,
			    XDM_HORTJUST, XDM_LEFT,
			    XDM_ARRAY, 1,
			    XDM_SPACE, 5, XDM_END));

    (void) buildNameValEtc(looksPtr, editField, "Priority:",
	    &(looksPtr->priorityField), NIL(struct bitmapEdit));

    (void) buildNameValEtc(looksPtr, editField, "Number of colors:",
	    &(looksPtr->nColorsField), NIL(struct bitmapEdit));

    rowField = buildNameValEtc(looksPtr, editField, "Current color:",
	    &(looksPtr->colorIndexField), NIL(struct bitmapEdit));
    XDM_ASSERT(XDMDialogMap(looksPtr->dWin, rowField,
						&(looksPtr->colorIndRow)));

    rowField = buildColor(looksPtr, editField, "Color:");
    XDM_ASSERT(XDMDialogMap(looksPtr->dWin, rowField, &(looksPtr->sliderRow)));

    (void) buildNameValEtc(looksPtr, editField, "Fill:",
	    &(looksPtr->fillField), &(looksPtr->fillBitmap));

    (void) buildNameValEtc(looksPtr, editField, "Border:",
	    &(looksPtr->borderField), &(looksPtr->borderBitmap));
}

static buildNameValEtc(looksPtr, field, name, fieldPtr, bitmapPtr)
struct layerLooks *looksPtr; int field;
char *name;
int *fieldPtr;
struct bitmapEdit *bitmapPtr;
{
    int rowField;
    int textField;
    int bitmapSize();

    rowField = XDMRowColCreate(looksPtr->dWin, field);
    XDM_ASSERT(XDMModify(looksPtr->dWin, rowField, XDM_ISROW, 1, XDM_END));

    textField = XDMTextCreate(looksPtr->dWin, rowField);
    XDM_ASSERT(XDMModify(looksPtr->dWin, textField, XDM_TEXT, name, XDM_END));

    *fieldPtr = XDMEdRegCreate(looksPtr->dWin, rowField);
    XDM_ASSERT(XDMModify(looksPtr->dWin, *fieldPtr,
			    XDM_ROWS, 1, XDM_COLS, 10,
			    XDM_USER, (XDMPointer) fieldPtr,
			    XDM_SETSIGNAL, 1, XDM_END));
    
    if (bitmapPtr) {
	(void) XDMForeignCreate(looksPtr->dWin, rowField, 1, 1, 1,
		BlackPixmap, WhitePixmap,
		bitmapSize, (int (*)()) 0, (int (*)()) 0, (int (*)()) 0,
		(int (*)()) 0, &(bitmapPtr->window));
	XMakeAssoc(looksTable, (XId) bitmapPtr->window, (caddr_t) looksPtr);
	XSelectInput(bitmapPtr->window, ButtonPressed | EnterWindow);
	bitmapPtr->bitIndexTable = XCreateAssocTable(1);
    }

    return(rowField);
}

static drawEdRegField(looksPtr, field)
struct layerLooks *looksPtr;
int field;
{
    int val;
    char string[50];

    if (field == looksPtr->priorityField) {
	val = looksPtr->priority;
    } else if (field == looksPtr->nColorsField) {
	val = looksPtr->nColors;
    } else if (field == looksPtr->colorIndexField) {
	if (looksPtr->currentColorIndex >= looksPtr->nColors) {
	    looksPtr->currentColorIndex = looksPtr->nColors - 1;
	}
	if (looksPtr->currentColorIndex < 0) looksPtr->currentColorIndex = 0;
	val = looksPtr->currentColorIndex;

	if (looksPtr->nColors > 0) {
	    XMapWindow(looksPtr->colorIndRow);
	    XMapWindow(looksPtr->sliderRow);
	    drawSlide(looksPtr->redSlide,
			(int) looksPtr->colorVals[val].red, redPixel.pixel);
	    drawSlide(looksPtr->greenSlide,
			(int) looksPtr->colorVals[val].green, greenPixel.pixel);
	    drawSlide(looksPtr->blueSlide,
			(int) looksPtr->colorVals[val].blue, bluePixel.pixel);
	} else {
	    XUnmapWindow(looksPtr->colorIndRow);
	    XUnmapWindow(looksPtr->sliderRow);
	    looksPtr->colorVals[0].red = 0;
	    looksPtr->colorVals[0].green = 0;
	    looksPtr->colorVals[0].blue = 0;
	}

	if (looksPtr->colorVals[val].pixel != BlackPixel) {
	    XStoreColor(&(looksPtr->colorVals[val]));
	}
    } else if (field == looksPtr->fillField) {
	if (looksPtr->fillSize < 0) looksPtr->fillSize = 0;
	val = looksPtr->fillSize;
	drawBitmap(&(looksPtr->fillBitmap), val, val);
    } else if (field == looksPtr->borderField) {
	if (looksPtr->borderSize < 0) looksPtr->borderSize = 0;
	val = looksPtr->borderSize;
	drawBitmap(&(looksPtr->borderBitmap), val, 1);
    }
    (void) sprintf(string, "%d", val);
    XDM_ASSERT(XDMModify(looksPtr->dWin, field, XDM_TEXT, string, XDM_END));
}

static updateEdRegField(looksPtr, field)
struct layerLooks *looksPtr;
int field;
{
    char *stringPtr;
    int newVal, i;

    XDM_ASSERT(XDMQuery(looksPtr->dWin, field, XDM_TEXT, &stringPtr, XDM_END));
    newVal = atoi(stringPtr);

    if (field == looksPtr->priorityField) {
	looksPtr->priority = newVal;
    } else if (field == looksPtr->nColorsField) {
	if (newVal < 0) newVal = 0;
	if (newVal > 0) {
	    looksPtr->colorVals = REALLOCn(newVal, Color, looksPtr->colorVals);
	    for (i = looksPtr->nColors; i < newVal; i++) {
		looksPtr->colorVals[i] = looksPtr->colorVals[0];
	    }
	} else {
	    looksPtr->colorVals = REALLOCn(1, Color, looksPtr->colorVals);
	}
	looksPtr->nColors = newVal;

	/* make sure currentColorIndex is in range */
	drawEdRegField(looksPtr, looksPtr->colorIndexField);
    } else if (field == looksPtr->colorIndexField) {
	looksPtr->currentColorIndex = newVal;
    } else if (field == looksPtr->fillField) {
	looksPtr->fillSize = atoi(stringPtr);
    } else if (field == looksPtr->borderField) {
	looksPtr->borderSize = atoi(stringPtr);
    }

    /* make sure the user sees what we think he typed */
    drawEdRegField(looksPtr, field);
}

static buildColor(looksPtr, field, name)
struct layerLooks *looksPtr;
int field;
char *name;
{
    int rowField;
    int textField;
    int colField;
    Window patchWindow;

    rowField = XDMRowColCreate(looksPtr->dWin, field);
    XDM_ASSERT(XDMModify(looksPtr->dWin, rowField, XDM_ISROW, 1, XDM_END));

    textField = XDMTextCreate(looksPtr->dWin, rowField);
    XDM_ASSERT(XDMModify(looksPtr->dWin, textField, XDM_TEXT, name, XDM_END));

    colField = XDMRowColCreate(looksPtr->dWin, rowField);

    buildSlideBar(looksPtr, colField, &(looksPtr->redSlide));
    buildSlideBar(looksPtr, colField, &(looksPtr->greenSlide));
    buildSlideBar(looksPtr, colField, &(looksPtr->blueSlide));

    (void) XDMForeignCreate(looksPtr->dWin, rowField,
	    PATCH_WIDTH, PATCH_HEIGHT, 1,
	    BlackPixmap, XMakeTile(looksPtr->colorVals[0].pixel),
	    (int (*)()) 0, (int (*)()) 0, (int (*)()) 0, (int (*)()) 0,
	    (int (*)()) 0, &patchWindow);

    return(rowField);
}

static buildSlideBar(looksPtr, colField, slideWinPtr)
struct layerLooks *looksPtr;
int colField;
Window *slideWinPtr;
{
    (void) XDMForeignCreate(looksPtr->dWin, colField,
		SLIDE_WIDTH, SLIDE_HEIGHT, 1,
		BlackPixmap, BlackPixmap,
		(int (*)()) 0, (int (*)()) 0, (int (*)()) 0, (int (*)()) 0,
		(int (*)()) 0, slideWinPtr);

    XMakeAssoc(looksTable, (XId) *slideWinPtr, (caddr_t) looksPtr);

    XSelectInput(*slideWinPtr, ExposeWindow |
		    ButtonPressed | LeftDownMotion | LeaveWindow);
}

static drawSlide(slideWindow, value, pixel)
Window slideWindow;
int value;
int pixel;
{
    value = ((SLIDE_WIDTH-1) * value) / 65535 + 1;

    XPixSet(slideWindow, 0, 0, value, SLIDE_HEIGHT, pixel);
    XPixSet(slideWindow, value, 0, SLIDE_WIDTH-value, SLIDE_HEIGHT, BlackPixel);

    XFlush();
}

static updateSlide(looksPtr, eventPtr, valuePtr, pixel)
struct layerLooks *looksPtr;
XMouseMovedEvent *eventPtr;
unsigned short *valuePtr;
int pixel;
{
    int x, y;
    Window subWin;

    if ((eventPtr->type & (ButtonPressed | MouseMoved | LeaveWindow)) &&
					    (eventPtr->detail & LeftMask)) {
	XUpdateMouse(eventPtr->window, &x, &y, &subWin);

	if (x >= SLIDE_WIDTH) x = SLIDE_WIDTH - 1;
	if (x < 0) x = 0;

	*valuePtr = (65535 * x) / (SLIDE_WIDTH-1);

	if (looksPtr->colorVals[looksPtr->currentColorIndex].pixel != BlackPixel) {
	    XStoreColor(&(looksPtr->colorVals[looksPtr->currentColorIndex]));
	}
    }

    drawSlide(eventPtr->window, (int) *valuePtr, pixel);
}

static drawBitmap(bitmapPtr, newW, newH)
struct bitmapEdit *bitmapPtr;
int newW, newH;
{
    int row, col;
    char *bitPtr;
    OpaqueFrame *frameList, *framePtr;
    WindowInfo winInfo;

    XDestroyAssocTable(bitmapPtr->bitIndexTable);
    XDestroySubwindows(bitmapPtr->window);

    /* copy bitmap, padding or truncating as necessary */
    bitPtr = ALLOCn(newW*newH + 1, char);
    for (row = 0; row < newH; row++) {
	for (col = 0; col < newW; col++) {
	    if (row < bitmapPtr->height && col < bitmapPtr->width) {
		bitPtr[row*newW + col] =
			bitmapPtr->bitString[row*(bitmapPtr->width) + col];
	    } else {
		bitPtr[row*newW + col] = '0';
	    }
	}
    }
    bitPtr[newW*newH] = '\0';
    FREE(bitmapPtr->bitString);
    bitmapPtr->bitString = bitPtr;

    bitmapPtr->width = newW;
    bitmapPtr->height = newH;

    bitmapPtr->bitIndexTable = XCreateAssocTable(assocTableSize(newW*newH));
    if (bitmapPtr->bitIndexTable == NIL(XAssocTable)) {
	ERROR("couldn't allocate bitmap assoc table");
    }

    XQueryWindow(bitmapPtr->window, &winInfo);
    if (newW == 0) {
	XUnmapWindow(bitmapPtr->window);
	XChangeWindow(bitmapPtr->window, 1, 1);
	XDMForeignResize(bitmapPtr->window, winInfo.width, winInfo.height,
		    1, 1);
	return;
    } else {
	XChangeWindow(bitmapPtr->window, newW*BIT_SIZE, newH*BIT_SIZE);
	XDMForeignResize(bitmapPtr->window, winInfo.width, winInfo.height,
		    newW*BIT_SIZE, newH*BIT_SIZE);
	XMapWindow(bitmapPtr->window);
    }

    framePtr = frameList = ALLOCn(newW*newH, OpaqueFrame);
    bitPtr = bitmapPtr->bitString;
    for (row = 0; row < newH; row++) {
	for (col = 0; col < newW; col++) {
	    framePtr->x = col * BIT_SIZE;
	    framePtr->y = row * BIT_SIZE;
	    framePtr->width = BIT_SIZE - 2;
	    framePtr->height = BIT_SIZE - 2;
	    framePtr->bdrwidth = 1;
	    if (*bitPtr++ == '1') {
		framePtr->background = BlackPixmap;
		framePtr->border = WhitePixmap;
	    } else {
		framePtr->background = WhitePixmap;
		framePtr->border = BlackPixmap;
	    }
	    framePtr++;
	}
    }

    if (XCreateWindows(bitmapPtr->window, frameList, newW*newH) != newW*newH) {
	ERROR("couldn't create enough windows for bitmap editing region");
    }
    XMapSubwindows(bitmapPtr->window);

    while (--bitPtr >= bitmapPtr->bitString) {
	XMakeAssoc(bitmapPtr->bitIndexTable,
				(XId) (--framePtr)->self, (caddr_t) bitPtr);
    }

    XFlush();
}

static bitmapSize(win, widthPtr, heightPtr)
Window win;
int *widthPtr, *heightPtr;
{
    struct layerLooks *looksPtr;

    looksPtr = (struct layerLooks *) XLookUpAssoc(looksTable, (XId) win);
    if (looksPtr == NIL(struct layerLooks)) {
	/* probably the window is just now being created */
	*widthPtr = *heightPtr = 3;
	return;
    }

    if (win == looksPtr->fillBitmap.window) {
	*widthPtr = looksPtr->fillBitmap.width * BIT_SIZE + 2;
	*heightPtr = looksPtr->fillBitmap.height * BIT_SIZE + 2;
    } else if (win == looksPtr->borderBitmap.window) {
	*widthPtr = looksPtr->borderBitmap.width * BIT_SIZE + 2;
	*heightPtr = looksPtr->borderBitmap.height * BIT_SIZE + 2;
    } else {
	ERROR("internal error -- bitmapSize called on unknown window");
    }
}

#define ON	0
#define OFF	1
#define TOGGLE	2

static updateBitmap(eventPtr, looksPtr, bitmapPtr)
XButtonEvent *eventPtr;
struct layerLooks *looksPtr;
struct bitmapEdit *bitmapPtr;
{
    char *bitPtr;
    int action;

    if (eventPtr->subwindow == 0) return;

    if (eventPtr->type == ButtonPressed) {
	switch (eventPtr->detail & ValueMask) {
	    case LeftButton:
		action = ON;
		break;
	    case MiddleButton:
		action = TOGGLE;
		break;
	    case RightButton:
		action = OFF;
		break;
	    default:
		return;
	}
    } else {
	if (eventPtr->detail & LeftMask) {
	    action = ON;
	} else if (eventPtr->detail & MiddleMask) {
	    action = TOGGLE;
	} else if (eventPtr->detail & RightMask) {
	    action = OFF;
	} else {
	    return;
	}
    }

    bitPtr = (char *) XLookUpAssoc(bitmapPtr->bitIndexTable,
						(XId) eventPtr->subwindow);
    if (bitPtr == NIL(char)) {
	ERROR("internal error: unknown subwindow of bitmap");
    }

    switch (action) {
	case ON:
	    *bitPtr = '1';
	    break;
        case OFF:
	    *bitPtr = '0';
	    break;
        case TOGGLE:
	    *bitPtr = '0' + '1' - *bitPtr;
	    break;
    }

    if (*bitPtr == '1') {
	    XChangeBackground(eventPtr->subwindow, BlackPixmap);
	    XChangeBorder(eventPtr->subwindow, WhitePixmap);
    } else {
	    XChangeBackground(eventPtr->subwindow, WhitePixmap);
	    XChangeBorder(eventPtr->subwindow, BlackPixmap);
    }

    XClear(eventPtr->subwindow);
    XFlush();

    drawPatch(looksPtr);
}

static buildPatchField(looksPtr, bigColumn)
struct layerLooks *looksPtr;
int bigColumn;
{
    (void) XDMForeignCreate(looksPtr->dWin, bigColumn, 50, 50, 0,
	    BlackPixmap, XMakeTile(looksPtr->bgPixel),
	    (int (*)()) 0, (int (*)()) 0, (int (*)()) 0, (int (*)()) 0,
	    (int (*)()) 0, &(looksPtr->patchWindow));

    XMakeAssoc(looksTable, (XId) looksPtr->patchWindow, (caddr_t) looksPtr);
    XSelectInput(looksPtr->patchWindow, ExposeWindow);
}

static buildButtonField(looksPtr, bigColumn)
struct layerLooks *looksPtr;
int bigColumn;
{
    int rowField;
    int textField;

    rowField = XDMRowColCreate(looksPtr->dWin, bigColumn);
    XDM_ASSERT(XDMModify(looksPtr->dWin, rowField, XDM_ISROW, 1, XDM_END));

    looksPtr->tryField = XDMButtonCreate(looksPtr->dWin, rowField);
    textField = XDMTextCreate(looksPtr->dWin, looksPtr->tryField);
    XDM_ASSERT(XDMModify(looksPtr->dWin, looksPtr->tryField,
			    XDM_SETSIGNAL, 1, XDM_END));
    XDM_ASSERT(XDMModify(looksPtr->dWin, textField,
			    XDM_TEXT, "Try", XDM_END));

    looksPtr->OKField = XDMButtonCreate(looksPtr->dWin, rowField);
    textField = XDMTextCreate(looksPtr->dWin, looksPtr->OKField);
    XDM_ASSERT(XDMModify(looksPtr->dWin, looksPtr->OKField,
			    XDM_SETSIGNAL, 1, XDM_END));
    XDM_ASSERT(XDMModify(looksPtr->dWin, textField,
			    XDM_TEXT, "OK", XDM_END));

    looksPtr->abortField = XDMButtonCreate(looksPtr->dWin, rowField);
    textField = XDMTextCreate(looksPtr->dWin, looksPtr->abortField);
    XDM_ASSERT(XDMModify(looksPtr->dWin, looksPtr->abortField,
			    XDM_SETSIGNAL, 1, XDM_END));
    XDM_ASSERT(XDMModify(looksPtr->dWin, textField,
			    XDM_TEXT, "Abort", XDM_END));
}

static Vertex patchBorder[] = {
    {		0,		0,	0 },
    {	PATCH_WIDTH-1,		0,	0 },
    {	PATCH_WIDTH-1,	PATCH_HEIGHT-1,	0 },
    {		0,	PATCH_HEIGHT-1,	0 },
    {		0,		0,	0 }
};
static drawPatch(looksPtr)
struct layerLooks *looksPtr;
{
    int inWidth, inHeight, outWidth, outHeight;
    int inRow, inCol, outRow, outCol, word, bitPos;
    char *bitString;
    unsigned short *bitField;
    Bitmap fillBM;
    Pixmap tile;
    int borderBits;
    Pattern borderPattern;

    inWidth = looksPtr->fillBitmap.width;
    inHeight = looksPtr->fillBitmap.height;
    bitString = looksPtr->fillBitmap.bitString;

    if (inWidth*inHeight == 0) {
	XClear(looksPtr->patchWindow);
    } else {
	XQueryTileShape(inWidth, inHeight, &outWidth, &outHeight);

	/* XXX unjustified assumption that a short is 16 bits */
	bitField = ALLOCn((outWidth*outHeight + 15) / 16, unsigned short);

	inRow = 0; word = 0;
	for (outRow = 0; outRow < outHeight; outRow++) {
	    inCol = 0; bitPos = 0;
	    for (outCol = 0; outCol < outWidth; outCol++) {
		if (bitPos == 16) {
		    bitPos = 0;
		    word++;
		}
		if (bitPos == 0) bitField[word] = 0;
		bitField[word] |=
			(bitString[inRow*inWidth + inCol] == '1') << bitPos++;
		if (++inCol == inWidth) inCol = 0;
	    }
	    if (++inRow == inHeight) inRow = 0;
	    word++;
	}

	fillBM = XStoreBitmap(outWidth, outHeight, bitField);
	if (fillBM == (Bitmap) 0) {
	    ERROR("couldn't get resources for patch bitmap");
	}

	tile = XMakePixmap(fillBM, looksPtr->colorVals[0].pixel,
				    looksPtr->bgPixel);
	if (tile == (Pixmap) 0) {
	    ERROR("couldn't get resources for patch tile pixmap");
	}

	XTileSet(looksPtr->patchWindow, 0, 0, PATCH_WIDTH, PATCH_HEIGHT, tile);

	FREE(bitField);
	XFreeBitmap(fillBM);
	XFreePixmap(tile);
    }

    outWidth = MIN(looksPtr->borderBitmap.width, 16);
    if (outWidth > 0) {
	borderBits = 0;
	for (outCol = 0; outCol < outWidth; outCol++) {
	    if (looksPtr->borderBitmap.bitString[outCol] == '1') {
		borderBits |= 1 << outCol;
	    }
	}
	borderPattern = XMakePattern(borderBits, outWidth, 1);
	XDrawDashed(looksPtr->patchWindow, patchBorder, 5, 1, 1,
		looksPtr->colorVals[0].pixel, borderPattern, GXcopy, AllPlanes);
    }
}

/*VARARGS*/
static void handleEvent()
{
    XEvent event;
    Window eventWin;
    int eventField;
    struct layerLooks *looksPtr;

    while (XPending() > 0) {
	XNextEvent(&event);
	if (XDMFilter(&event, &eventWin, &eventField) < 0) {
	    looksPtr = (struct layerLooks *) XLookUpAssoc(looksTable,
							(XId) eventWin);
	    switch (XDMTypeQuery(eventWin, eventField)) {
		case XDM_BTN_OBJ:
		    handleButtonEvent(looksPtr, eventField);
		    break;
		case XDM_EDREG_OBJ:
		    updateEdRegField(looksPtr, eventField);
		    break;
		default:
		    break;
	    }
	} else {
	    looksPtr = (struct layerLooks *) XLookUpAssoc(looksTable,
							(XId) event.window);
	    if (looksPtr) {
		handleForeignEvent(looksPtr, &event);
	    }
	}
    }
}

static handleButtonEvent(looksPtr, fieldId)
struct layerLooks *looksPtr;
int fieldId;
{
    if (fieldId == looksPtr->abortField) {
	destroyLooksEditor(looksPtr);
    } else if (fieldId == looksPtr->tryField) {
	convertLooksToLayer(looksPtr);
	XDM_ASSERT(XDMModify(looksPtr->dWin, fieldId, XDM_VALUE, 0, XDM_END));
    } else if (fieldId == looksPtr->OKField) {
	convertLooksToLayer(looksPtr);
	destroyLooksEditor(looksPtr);
    }
}

static handleForeignEvent(looksPtr, eventPtr)
struct layerLooks *looksPtr;
XEvent *eventPtr;
{
    if (eventPtr->window == looksPtr->redSlide) {
	updateSlide(looksPtr, (XMouseMovedEvent *) eventPtr,
		    &(looksPtr->colorVals[looksPtr->currentColorIndex].red),
		    redPixel.pixel);
    } else if (eventPtr->window == looksPtr->greenSlide) {
	updateSlide(looksPtr, (XMouseMovedEvent *) eventPtr,
		    &(looksPtr->colorVals[looksPtr->currentColorIndex].green),
		    greenPixel.pixel);
    } else if (eventPtr->window == looksPtr->blueSlide) {
	updateSlide(looksPtr, (XMouseMovedEvent *) eventPtr,
		    &(looksPtr->colorVals[looksPtr->currentColorIndex].blue),
		    bluePixel.pixel);
    } else if (eventPtr->window == looksPtr->fillBitmap.window) {
	updateBitmap((XButtonEvent *) eventPtr, looksPtr,
						&(looksPtr->fillBitmap));
    } else if (eventPtr->window == looksPtr->borderBitmap.window) {
	updateBitmap((XButtonEvent *) eventPtr, looksPtr,
						&(looksPtr->borderBitmap));
    } else if (eventPtr->window == looksPtr->patchWindow) {
	drawPatch(looksPtr);
    }
}
#endif
