#include <stdio.h>
#include "tech-edit.h"
#include "utility.h"
#include "tap.h"

printError(msg)
char *msg;
{
    ITALICS(msg);
    vemPrompt();
}

char *checkPtr(ptr)
char *ptr;
{
    if (ptr == NIL(char)) {
	printError("Ran out of memory in tech-edit -- good-bye");
	RPCExit(-1);
    }
    return(ptr);
}

/*
*	return a good number of bins for an XAssocTable of 'elts' elements
*/
assocTableSize(elts)
int elts;
{
    int bins;

    elts /= 8;

    for (bins = 1; bins <= elts; bins <<= 1) {
    }

    return(bins);
}

checkTechProp()
{
    octObject prop;

    if (!techPropOK) {
	if (dmConfirm("Wrong Value of TECHNOLOGY Property",
				    "Correct It?", "Yes", "No") == VEM_OK) {
	    prop.type = OCT_PROP;
	    prop.contents.prop.name = TECH_PROP_NAME;
	    prop.contents.prop.type = OCT_STRING;
	    prop.contents.prop.value.string = TAP_TECH_PROP_VAL;
	    OCT_ASSERT(octCreateOrModify(&theFacet, &prop));
	    techPropOK = 1;
	    vemFlushTechInfo(&theFacet);
	    tapFlushCache();
	} else {
	    ERROR("Command aborted");
	}
    }
}

findRules(spotPtr, cmdList, itemListPtr)
RPCSpot *spotPtr;
lsList cmdList;
dmWhichItem **itemListPtr;
{
    int numFound;
    lsList newCmdList;
    struct RPCArg newArg;
    struct RPCPointArg newPoint;

    if (lsLength(cmdList) == 0) {
	newPoint.theWin = spotPtr->theWin;
	newPoint.facet = spotPtr->facet;
	newPoint.thePoint = spotPtr->thePoint;

	newArg.argType = VEM_POINT_ARG;
	newArg.length = 1;
	newArg.argData.points = &newPoint;

	newCmdList = lsCreate();
	lsNewBegin(newCmdList, (Pointer) &newArg, (lsHandle *) 0);

	numFound = doFindRules(newCmdList, itemListPtr);

	lsDestroy(newCmdList, (void (*)()) 0);
    } else {
	numFound = doFindRules(cmdList, itemListPtr);
    }

    return(numFound);
}

doFindRules(cmdList, itemListPtr)
lsList cmdList;
dmWhichItem **itemListPtr;
{
    octObject bigBag, ruleBag;
    octGenerator gen1, gen2;
    octStatus status;
    octObject path;
    struct octBox bbox;
    static dmWhichItem *itemList = NIL(dmWhichItem);
    int numFound = 0;
    char ruleName[500];
    octCoord ruleDist;
    octCoord tapEvaluateDesignRule();

    bigBag.type = OCT_BAG;
    bigBag.contents.bag.name = BIG_BAG_NAME;
    OCT_ASSERT(octGetOrCreate(&theFacet, &bigBag));

    OCT_ASSERT(octInitGenContents(&bigBag, OCT_BAG_MASK, &gen1));
    while ((status = octGenerate(&gen1, &ruleBag)) == OCT_OK) {
	OCT_ASSERT(octInitGenContents(&ruleBag, OCT_PATH_MASK, &gen2));
	while ((status = octGenerate(&gen2, &path)) == OCT_OK) {
	    OCT_ASSERT(octBB(&path, &bbox));
	    if (inSelectedArea(&bbox, cmdList)) {
		numFound++;
		itemList = REALLOC(dmWhichItem, itemList, numFound);
		ruleDist = tapEvaluateDesignRule(&ruleBag, ruleName);
		if (ruleDist == TAP_RULE_ERROR) {
		    (void) strcpy(ruleName, "<error>");
		} else {
		    (void) sprintf(&ruleName[strlen(ruleName)],
					" (%d)", ruleDist);
		}
		itemList[numFound-1].itemName = util_strsav(ruleName);
		itemList[numFound-1].userData = (Pointer) ruleBag.objectId;
	    }
	}
	OCT_ASSERT(status);
    }
    OCT_ASSERT(status);

    *itemListPtr = itemList;
    return(numFound);
}

freeRuleList(itemList, count)
dmWhichItem *itemList;
int count;
{
    int i = 0;
    while (count--) {
	FREE( itemList[i].itemName );
	i++;
    }
}

findDevices(itemListPtr)
dmWhichItem **itemListPtr;
{
    octGenerator layerGen, deviceGen;
    octObject layer, dispBag, deviceBag, colorProp;
    octStatus status;
    int numFound = 2;
    dmWhichItem *itemList = ALLOC(dmWhichItem, 2);
    int i;

    /* fill in the two default names */
    itemList[0].itemName = DEFAULT_COLOR_NAME;
    itemList[0].userData = (Pointer) TAP_COLOR;
    itemList[1].itemName = DEFAULT_BW_NAME;
    itemList[1].userData = (Pointer) TAP_BW;

    OCT_ASSERT(octInitGenContents(&theFacet, OCT_LAYER_MASK, &layerGen));
    while ((status = octGenerate(&layerGen, &layer)) == OCT_OK) {
	dispBag.type = OCT_BAG;
	dispBag.contents.bag.name = DISPLAY_BAG_NAME;
	if (octGetByName(&layer, &dispBag) != OCT_OK) continue;

	OCT_ASSERT(octInitGenContents(&dispBag, OCT_BAG_MASK, &deviceGen));
	while ((status = octGenerate(&deviceGen, &deviceBag)) == OCT_OK) {
	    for (i = 0; i < numFound; i++) {
		if (strcmp(deviceBag.contents.bag.name, itemList[i].itemName)
									== 0) {
		    break;
		}
	    }
	    if (i >= numFound) {
		numFound++;
		itemList = REALLOC(dmWhichItem, itemList, numFound);
		itemList[i].itemName = deviceBag.contents.bag.name;
		colorProp.type = OCT_PROP;
		colorProp.contents.prop.name = COLOR_NAME;
		itemList[i].userData =
			(octGetByName(&deviceBag, &colorProp) == OCT_OK) ?
			    (Pointer) TAP_COLOR : (Pointer) TAP_BW;
	    }
	}
	OCT_ASSERT(status);
    }
    OCT_ASSERT(status);

    numFound++;
    itemList = REALLOC(dmWhichItem, itemList, numFound);
    itemList[numFound-1].itemName = util_strsav("(other)");

    *itemListPtr = itemList;

    return(numFound);
}

inSelectedArea(boxPtr, cmdList)
struct octBox *boxPtr;
lsList cmdList;
{
    lsGen cmdGen;
    RPCArg *argPtr;
    lsStatus status;
    int index;
    int count;
    struct octBox argBox;

    cmdGen = lsStart(cmdList);
    while ((status = lsNext(cmdGen, (Pointer *) &argPtr, NIL(lsHandle))) !=
								LS_NOMORE) {
	if (status != LS_OK) {
	    ERROR("\nInternal error scanning RPC args\n");
	}

	switch (argPtr->argType) {
	    case VEM_POINT_ARG:
	    case VEM_LINE_ARG:
	    case VEM_BOX_ARG:
		break;
	    default:
		ERROR("\nOnly points, lines, and boxes allowed as arguments\n");
	}

	index = 0;
	for (count = 0; count < argPtr->length; count++) {
	    if (argPtr->argData.points[index].facet != theFacet.objectId) {
		ERROR("\nPoints must be in facet being edited\n");
	    }
	    switch (argPtr->argType) {
		case VEM_POINT_ARG:
		    argBox.lowerLeft = argBox.upperRight =
				    argPtr->argData.points[index++].thePoint;
		    if (BOX_INTERSECT(*boxPtr, argBox)) {
			lsFinish(cmdGen);
			return(1);
		    }
		    break;
		case VEM_LINE_ARG:
		    if (count == 0) count++;
		    argBox.lowerLeft = argPtr->argData.points[index++].thePoint;
		    argBox.upperRight = argPtr->argData.points[index].thePoint;
		    BOX_NORM(argBox);
		    if (BOX_INTERSECT(*boxPtr, argBox)) {
			lsFinish(cmdGen);
			return(1);
		    }
		    break;
		case VEM_BOX_ARG:
		    argBox.lowerLeft = argPtr->argData.points[index++].thePoint;
		    argBox.upperRight = argPtr->argData.points[index++].thePoint;
		    BOX_NORM(argBox);
		    if (BOX_INSIDE(*boxPtr, argBox)) {
			lsFinish(cmdGen);
			return(1);
		    }
		    break;
	    }
	}
    }

    lsFinish(cmdGen);
    return(0);
}
