/*
 * This program translates a old style VEM patterns file into 
 * a tap view.
 *
 * David Harrison
 * University of California,  1987
 */

#include "copyright.h"
#include "port.h"
#include <X11/Xlib.h>
#include "utility.h"
#include "oct.h"
#include "oh.h"
#include "options.h"
#include "tap.h"
#include "th.h"

char *strnolen = "";

char errBuf[4096];
char *errMsgArea = errBuf;


#define INPUT_LEN	1024

#define PRINTF	(void) printf


void doRegularFile( techFileName, tech, view )
    char* techFileName;
    char* tech;
    char *view;
{
    if ( techFileName ) {
	
	octObject facet;
	tapOpenTechFacet( tech, view, &facet, "a" );
	thTechnology( &facet );
	thReadFile( techFileName );
	OH_ASSERT(octFlushFacet( &facet ));
	octEnd();
	exit ( 0 );
    }
}






optionStruct optionList[] = {
    {"t", "name",   "TAP technology name: defaults to scmos" },
    {"v", "name",         "TAP view name: defaults to physical" },
    {"O", "name", "Output Display type (can be determined automatically)"},
    {"C", 0,                    "Use this option if output display has color"},
    {"L", "",		       "The input file is a layer file (old VEM format)" },
    {"T", "tech name",         "Technology in pattern file (e.g physical, symbolic)" },
    {"D", "name",              "Delete the specified display type from TAP" },
    {"l", 0,                   "List existing display types in TAP" },
    {OPT_RARG, "tech file name",    "File with technological info" },
   { 0, 0 , 0}
};



main(argc, argv)
int argc;
char *argv[];
{
    char *pat_file;
    char *root_dir = tapGetDefaultRoot();
    char *out_tech = "zombie";
    char *out_view = "physical";
    char *out_disp = 0;
    char *disp_inp;
    char *disp_name = 0;
    char *chr;
    int chroma;
    Display *display;
    int c;
    int doPatternFile = 0;	/* Flag */
    char* techFileName = 0;	/*  */
    char *tech_spec = "physical";
    int hasColor = 0;
    char *deleteDisplayType = 0;
    int listDisplayTypes = 0;
    

    while ((c = optGetOption(argc,argv)) != EOF) {
	switch(c) {
	case 't': out_tech = util_strsav( optarg ); break;
	case 'T': tech_spec = util_strsav( optarg ); break;
	case 'v': out_view = util_strsav( optarg ); break;
	case 'O': out_disp = util_strsav( optarg ); break;
	case 'C': hasColor = 1;	break;
	case 'L': doPatternFile = 1; break;

	case 'D': deleteDisplayType = util_strsav( optarg ); break;
	case 'l': listDisplayTypes = 1; break;


	default:
	    optUsage();
	    break;
	}
    }

    if ( argc == 1 || optind != argc-1) optUsage();

    techFileName = util_strsav( argv[optind] );


    if ( deleteDisplayType != 0 ) {
	
	deleteDispType( out_tech, out_view, deleteDisplayType );
	exit(0);
    } 

    if ( listDisplayTypes ) {
	listAllDisplayTypes(out_tech, out_view);
	exit(0);
    }

    if ( !doPatternFile ) {
	doRegularFile( techFileName, out_tech, out_view );
    } else {

	root_dir = tapGetDefaultRoot();
	pat_file = techFileName;

	if ( out_disp == 0 ) {
	    /* Try to find out the display type from the display name */
	    if ( disp_name == 0 ) {
		disp_name = getenv("DISPLAY");
	    } 
	    if ( disp_name != 0 ) {
		display = XOpenDisplay(disp_inp);
		out_disp = ServerVendor(display);
	    } else {
		out_disp = "GENERIC-COLOR";
	    }

	    /* chr = prompt("Color output device",
	       ((DisplayCells(display, DefaultScreen(display)) > 15) ? "yes" : "no")); */
	 
	}
	chroma = hasColor ? TAP_COLOR  : TAP_BW ;

	
		
	process(pat_file, tech_spec, root_dir, out_tech, out_view, out_disp, chroma);
	octEnd();

    }
    
    exit( 0 );
}



/*ARGSUSED*/
int vemMessage(str, directive)
char *str;
int directive;
/*
 * Quick replacement for vemMessage
 */
{
    PRINTF(str);
    return 1;
}



int stricmp(a, b)
register char *a, *b;
/*
 * This routine compares two strings disregarding case.
 */
{
    register int value;

    if ((a == (char *) 0) || (b == (char *) 0)) {
	return a - b;
    }

    for ( /* nothing */;
	 ((*a | *b) &&
	  !(value = ((isupper(*a) ? *a - 'A' + 'a' : *a) -
		     (isupper(*b) ? *b - 'A' + 'a' : *b))));
	 a++, b++)
      /* Empty Body */;

    return value;
}
